/*
 * Copyright (C) 2008-2010 WaveMaker Software, Inc.
 *
 * This file is part of WaveMaker Studio.
 *
 * WaveMaker Studio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, version 3 of the License, only.
 *
 * WaveMaker Studio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WaveMaker Studio.  If not, see <http://www.gnu.org/licenses/>.
 */ 
dojo.provide("wm.studio.app.project");

dojo.declare("wm.studio.Project", null, {
	//=========================================================================
	// New
	//=========================================================================
	newProject: function(inName) {
		var n = inName || this.projectName || "Project";
		this.projectName = wm.getValidJsName(n);
		this.pageName = "Main";
		var d = studio.studioService.requestAsync("newProject", [this.projectName]);
		d.addCallbacks(dojo.hitch(this, this.finishNewProject), function(inResult) {
			console.log("New Project Failed: ", inResult);
		});
		return d;
	},
	finishNewProject: function(inResult) {
		this.projectChanging();
		this.createApplicationArtifacts();
		this.makeApplication();
		this.newPage(this.pageName);
		this.saveProject(this.projectName);
		this.projectChanged();
		this.projectsChanged();
		//studio.deploy("Configuring Project...");
	},
	newPage: function(inName) {
		if (!this.projectName)
			return;
		this.pageChanging();
		this.pageName = inName;
		this.createPageArtifacts();
		this.makePage();
		this.savePage();
		this.pageChanged();
		this.pagesChanged();
	},
	createApplicationArtifacts: function() {
		var ctor = dojo.declare(this.projectName, wm.Application);
		this.projectData = {css: ""};
	},
	createPageArtifacts: function() {
		var ctor = dojo.declare(this.pageName, wm.Page);
		this.pageData = {};
	        this.pageData.widgets = ctor.widgets = this.getPageTemplate();
	        studio.setScript(this.pageData.js = pageScript(this.pageName));
		this.pageData.css = this.pageData.html = "";
	},
	getPageTemplate: function() {
		// NOTE: could present list of choices here
		return {layoutBox1: ["wm.Layout", {height: "100%", width: "100%", horizontalAlign: "left", verticalAlign: "top"}, {}, {}]};
	},
	//=========================================================================
	// Open
	//=========================================================================		 
	openProject: function(inProjectName, inPageName) {
		if (!inProjectName) {
			this.closeProject();
			return;
		}

		if (this.projectName && this.projectName != null && this.projectName != '')
		{
			if (this.projectName != inProjectName)
			{
				this.closeProject(this.projectName);
			}
		}
		
		var o = studio.studioService.getResultSync("openProject", [inProjectName]);
	        if (o) {
			this.projectName = inProjectName;
			if (o.upgradeMessages)
				this.showUpgradeMessage(o.upgradeMessages);
			this.projectChanging();
			try {
				this.loadApplication();
				var ctor = dojo.getObject(this.projectName);
				this.pageName = inPageName || (ctor ? ctor.prototype.main : "Main");
				this.makeApplication();
				this.openPage(this.pageName);
			} catch(e) {
				console.debug(e);
				this.loadError("Failed to open project " + this.projectName + ". Error: " + e);
				this.projectName = "";
				this.pageName = "";
				studio.application = studio.page = null;
			} finally {
				this.projectChanged();
			}
		}
		
		// If we failed to open a project, then we still have sent a message to server closing the current project so we need to close it in studio
		// as well, and get back to the start screen.  Failure to do this will cause studio and server to be out of sync on what project is open.
		else {
		    alert(studio.studioService.error || "Failed to open project");
		    this.closeProject();
		}
	},
	openPage: function(inName) {
		this.pageChanging();
		this.pageName = inName;
		try {
			this.loadPage();
			this.makePage();
		} catch(e) {
			console.debug(e);
			this.loadError("Failed to open page " + this.pageName + ". Error: " + e);
			this.pageName = "";
			studio.page = null;
		} finally {
			this.pageChanged();
		}
	},
	loadProjectData: function(inPath) {
	    //return wm.load("projects/" + studio.projectPrefix + this.projectName + "/" + inPath);
		     return wm.load("projects/" + this.projectName + "/" + inPath);
	},
	loadApplication: function() {
		this.projectData = {
			js: this.loadProjectData(this.projectName + ".js"),
			css: this.loadProjectData("app.css")
		};
		if (!this.projectData.js) {
		    alert("Warning: Could not find projects/" + this.projectName + "/" + this.projectName + ".js");
		}
		eval(this.projectData.js);
	},
	loadPage: function() {
		var n = this.pageName, f = wm.pagesFolder + n + "/", p = f + n ;
		this.pageData = {
			js: this.loadProjectData(p + ".js"),
			widgets: this.loadProjectData(p + ".widgets.js"),
			css: this.loadProjectData(p + ".css"),
			html: this.loadProjectData(p + ".html")
		}
		var ctor = dojo.declare(n, wm.Page);
		eval(this.pageData.widgets);
	},
	makeApplication: function() {
		var ctor = dojo.getObject(this.projectName);
		if (ctor)
			studio.application = new ctor({ _designer: studio.designer });
	},
	makePage: function() {
		var ctor = dojo.getObject(this.pageName);
		if (ctor) {
			studio.connect(ctor.prototype, "init", function() { studio.page = this; });
			studio.connect(ctor.prototype, "start", function() { wm.fire(studio.application, "start"); });
			studio.page = new ctor({
				name: "wip",
				domNode: studio.designer.domNode,
				owner: studio,
				_designer: studio.designer
			});
			studio.page.root.parent = studio.designer;
		}
	},
	loadError: function(inMessage) {
		alert(inMessage);
		this.projectChanged();
	},
	//=========================================================================
	// Save
	//=========================================================================
	saveProject: function() {
		this.saveApplication();
		this.savePage();
	},
	saveScript: function() {
  	       //this.savePageData(this.pageName + ".js", studio.getScript());
		 this.saveProject();
	},
	saveCss: function() {
	       //this.savePageData(this.pageName + ".css", studio.getCss());
		 this.saveProject();
	},
	saveMarkup: function() {
   	      //this.savePageData(this.pageName + ".html", studio.getMarkup());
		 this.saveProject();
	},
        getProjectPath: function() {
	    //return studio.projectPrefix + this.projectName;
	     return this.projectName;
	},
	saveApplication: function() {
		var src = this.generateApplicationSource()
		this.saveProjectData(this.projectName + ".js", src);
		// save html file, config file, and debug loader + css
		var c = wm.studioConfig;
		this.saveProjectData(c.appIndexFileName, makeIndexHtml(this.projectName), true);

		/* 
		// Fix config.js if there is a username associated with this project (projectdir contains username or is empty string)
		if (window.studio && studio.getProjectDir() &&  c.appConfigTemplate.indexOf('"../wavemaker/') != -1) {
		    c.appConfigTemplate = c.appConfigTemplate.replace('"../wavemaker/', '"../../wavemaker/');
		}	
		*/	
		this.saveProjectData(c.appConfigFileName, c.appConfigTemplate, true);

		this.saveProjectData(c.appCssFileName, studio.getAppCss());
		this.saveProjectData(c.appDebugBootFileName, '', true);
		if (wm.studioConfig.isPalmApp) {
			this.saveProjectData(c.appPalmAppInfoFileName, makePalmAppInfo(this.projectName), true);
			this.saveProjectData("app/views/first/first-scene.html", "", true);
		}
		studio.setCleanApp();
	},
	generateApplicationSource: function() {
		var main = studio.application.main || "Main";
		var appClassName = (wm.studioConfig.isPalmApp ? "wm.PalmApplication" : "wm.Application");
		var src = [
			'dojo.declare("' + this.projectName + '", ' + appClassName + ', {',
				'\tmain: "' + main + '",',
				'\twidgets: {',
				studio.application.write(sourcer_tab + sourcer_tab),
				'\t}',
			'});'
		].join(sourcer_nl);
		return src;
	},
	savePageAs: function(inName) {
		var previousName = this.pageName;
		this.pageName = inName;
		// if name changed update source
		if (this.pageName != previousName)
			studio.pageNameChange(previousName, this.pageName);
		this.savePage();
		this.pageChanged();
		this.pagesChanged();
		this.openPage(this.pageName);
	},
	savePage: function() {
		this.savePageData(this.pageName + ".js", studio.getScript());
		this.savePageData(this.pageName + ".widgets.js", studio.getWidgets());
		this.savePageData(this.pageName + ".css", studio.getCss());
		this.savePageData(this.pageName + ".html", studio.getMarkup());
		studio.setCleanPage();
	},
	saveProjectData: function(inPath, inData, inNoOverwrite) {
		return studio.studioService.requestSync("writeWebFile", [inPath, inData, inNoOverwrite||false]);
	},
	savePageData: function(inPath, inData, inNoOverwrite) {
		var path = wm.pagesFolder + this.pageName + "/" + inPath;
		return studio.studioService.requestSync("writeWebFile", [path, inData, inNoOverwrite||false]);
	},
	copyProject: function(inName, inNewName) {
		if (inName && inNewName)
			this.saveProject();
			studio.studioService.requestAsync("copyProject", [inName, inNewName], dojo.hitch(this, "projectsChanged"));
	},
	//=========================================================================
	// Delete
	//=========================================================================
	deleteProject: function(inName) {	  
 	        var deleteCurrentProject = (inName == this.projectName);
		if  (deleteCurrentProject)
			this.closeProject();		
		return studio.studioService.requestAsync("deleteProject", [inName], dojo.hitch(this, function() {
			this.projectChanged(inName);
		}));
	},
	deletePage: function(inName) {
		return studio.pagesService.requestAsync("deletePage", [inName],
			dojo.hitch(this, function() {
				if (inName == this.pageName)
					this.openPage(studio.application.main);
				this.pagesChanged();
			}),
			function(inError) {
				alert(inName + " could not be deleted, error: " + inError);
			}
		);
	},
	//=========================================================================
	// Close
	//=========================================================================
	closeProject: function(inProjectName) {
		wm.fire(studio._deployer, "cancel");
		this.pageChanging();
		this.projectChanging();
		studio.studioService.requestSync("closeProject");
		this.pageList = [];
		this.projectName = this.pageName = "";
		this.pageChanged();
		this.projectChanged(inProjectName);
	},
	//=========================================================================
	// Project info / util
	//=========================================================================
	showUpgradeMessage: function(inMessages) {
		var message = "Your project has been upgraded.  A backup of your old project is available at:\n"+inMessages.backupExportFile;
		if (inMessages.messages) {
			var firstElem = true;
			for (var key in inMessages.messages) {
				if (firstElem) {
					message += "\n\nImportant messages regarding your upgrade:\n";
					firstElem = false;
				}
				message += "\t" + inMessages.messages[key] + "\n";
			}
		}
		alert(message);
	},
	updatePageList: function() {
		var d = studio.pagesService.requestSync("listPages", null);
		d.addCallback(dojo.hitch(this, function(inResult) {
			inResult.sort();
			this.pageList = inResult;
			return inResult;
		}));
		return d;
	},
	getPageList: function() {
		return this.pageList || [];
	},
	updateProjectList: function() {
		var d = studio.studioService.requestSync("listProjects", null);
		d.addCallback(dojo.hitch(this, function(inResult) {
			inResult.sort();
			this.projectList = inResult;
			return inResult;
		}));
		return d;
	},
	getProjectList: function() {
		return this.projectList || [];
	},
	projectsChanged: function() {
		studio.projectsChanged();
	},
	pagesChanged: function() {
		this.updatePageList();
		studio.pagesChanged();
	},
	projectChanging: function() {
		this.updatePageList();
		studio.projectChanging();
	},
	projectChanged: function(optionalInName) {
	        var name = (optionalInName || this.projectName);
		var projectData = (name == this.projectName) ? (this.projectData || {}) : {};
		studio.projectChanged(name, projectData);
	},
	pageChanging: function() {
		studio.pageChanging();
	},
	pageChanged: function() {
		studio.pageChanged(this.pageName, this.pageData || {});
	}
});

//=========================================================================
// Project Clean / Dirty
//=========================================================================
Studio.extend({
	setCleanPage: function(inPageData) {
		this._cleanPageData = this.page ? {
			js: inPageData ? inPageData.js : this.getScript(),
			widgets: inPageData ? inPageData.widgets : this.getWidgets()
		} : {};
	},
	setCleanApp: function() {
		this._cleanAppData = this.application ? { script: this.project.generateApplicationSource() } : {};
	},
	isPageDirty: function() {
		var c = this._cleanPageData;
		if (!c)
			return;
		return this.page && (c.js != this.getScript() || c.widgets != this.getWidgets());
	},
	isAppDirty: function() {
		var c = this._cleanAppData;
		if (!c)
			return;
		return this.application && (c.script != this.project.generateApplicationSource());
	},
	isProjectDirty: function() {
		return this.isAppDirty() || this.isPageDirty();
	}
});

//=========================================================================
// Project UI
//=========================================================================
Studio.extend({
	newPageClick: function() {
		if (!this.project.projectName)
			return;
		if (!this.isShowingWorkspace())
			this.tabs.setLayer("workspace");
		if (!this.confirmPageChange("Are you sure you want to add a new page? Unsaved changes to \"${page}\" will be lost."))
			return;
		var pages = this.project.getPageList();
		var l = {};
		dojo.forEach(pages, function(p) {
			l[p] = true;
		});
	        var n = wm.capitalize(this.promptForName("page", wm.findUniqueName('Page', [l]), pages));
		if (n)
			this.waitForCallback("Creating page: "+ n, dojo.hitch(this.project, "newPage", n));
	},
	newProjectClick: function() {
		if (this.project.projectName) {
			if (!this.confirmAppChange("プロジェクト「${project}」を閉じますか？保存されていない内容はなくなります。"))
				return;
		}
		var projects = this.project.getProjectList();
		var l={};
		dojo.forEach(projects, function(p) {
			l[p] = true;
		});
		var n = this.promptForName("project", wm.findUniqueName('Project', [l]), projects);
		if (n) {
			studio.studioService.requestSync("closeProject");
			this.waitForCallback("Creating project: "+ n, dojo.hitch(this.project, "newProject", n));
		}
	},
	welcomeOpenClick: function() {
		this.projects.activate();
	},
	saveProjectClick: function() {
		this.waitForCallback("Saving project: "+ this.project.projectName, dojo.hitch(this.project, "saveProject"));
	},
	saveScriptClick: function() {
		this.waitForCallback("Saving script: "+ this.project.projectName, dojo.hitch(this.project, "saveScript"));
	},
	saveCssClick: function() {
		this.waitForCallback("Saving CSS: "+ this.project.projectName, dojo.hitch(this.project, "saveCss"));
	},
	saveMarkupClick: function() {
		this.waitForCallback("Saving Markup: "+ this.project.projectName, dojo.hitch(this.project, "saveMarkup"));
	},
	savePageAsClick: function() {
		var n = this.promptForName("page", this.page.declaredClass, this.project.getPageList());
		if (n)
			this.waitForCallback("Saving page as: "+ n, dojo.hitch(this.project, "savePageAs", n));
	},
	importJavascriptLibrary: function() {
	    this.beginBind("Script Importer", studio.editArea, "js");
	},
	importCssLibrary: function() {
	    this.beginBind("Css Importer", studio.cssEditArea, "css");
	},
	beginBind: function(inPropName, editArea, type) {
	    var bd = this.getBindDialog();
		    //p = this.getBindDialogProps(inPropName),
	    var  p = {targetProperty: inPropName,
		      object: editArea // random object
		     };

	    bd.positionLocation = "bl";
	    bd.positionNode = editArea;
	    if (p) {
		bd.page.update(p);
		// FIXME: wm.calcOffset fails for td/tr so use tr for now.

		bd.bindSourceDialog.resourceRb.editor.setChecked(true);
		bd.bindSourceDialog.treeControlsPanel.hide();
		bd.bindSourceDialog.applyButton.setCaption("Import");
		bd.bindSourceDialog.applyButtonClick = function() {
		    var filepath = bd.bindSourceDialog.bindEditor.getValue("dataValue");
		    var newtext = "";
		    if (type == "css")
			newtext = "@import \"" + filepath + "\";";
		    else
		       newtext = "eval(wm.load(\"" + filepath + "\"));";

		    editArea.setText(newtext + "\n" + editArea.getText());
		    bd.bindSourceDialog.cancelButtonClick();
		};

		bd.show();
		bd.domNode.style.display = "block"; // Obviously I'm doing something wrong; this shouldn't be needed
		return true;
	    }
	},
	endBind: function(inPropName, inNode) {
	    if (this.bindDialog) {
		this.bindDialog.destroy();
		this.bindDialog = undefined;
	    }
	},
	getBindDialog: function() {

		var
		    props = {
			owner: this,
			pageName: "BindSourceDialog",
			scrimBackground: true,
			hideOnClick: false,
			positionLocation: "tl",
			border: "1px"
		    },
		    d = this.bindDialog = new wm.PagePopup(props);
		    d.setContainerOptions(true, 450, 300);
	    var b = this.bindDialog;

	    if (b._hideConnect)
		dojo.disconnect(b._hideConnect);
	    b._hideConnect = dojo.connect(b, "onHide", this, "endBind");
	    return b;
	},
	deleteSelectedProjectPageClick: function(inSender) {
		var n = this.projectsTree.selected;
		if (!n)
			return;
		if (n.page)
			this.deletePage(n.page);
		else if (n.project)
			this.deleteProject(n.project);
	},
	openSelectedProjectPageClick: function(inSender) {
		var n = this.projectsTree.selected;
		if (!n)
			return;
		var
			page = n.page, project = n.project,
			warnPage = "プロジェクト「${newPage}」を開きますか？「${page}」で保存されていない内容はなくなります。",
			warnProject = "プロジェクト「${project}」を閉じますか？保存されていない内容はなくなります。";
		if (project == this.project.projectName) {
			if (page && page != this.project.pageName && this.confirmPageChange(warnPage, page)) {
				this.waitForCallback("Opening page: " + page + ".", dojo.hitch(this.project, "openPage", page));
			}
		} else if (project && this.confirmAppChange(warnProject)) {
			studio.studioService.requestSync("closeProject");
			var p = "プロジェクトを開いています： " + project + (page ? ", and page " + page : "") + ".";
			this.waitForCallback(p, dojo.hitch(this.project, "openProject", project, page));
		}
	},
	openProjectClick: function() {
		this.navGotoEditor(this.startLayer.name);
		this.startEditor.page.openProjectTab();
	},
	deleteProjectClick: function() {
	  if (studio.project) {
	    this.deleteProject(studio.project.projectName);
	  }
	},
	deleteProject: function(inName) {
		var p = inName;
		if (p && confirm(p + 'を削除しますか？')) {
		        if (studio.project.projectName == inName)
			     this.project.closeProject();
			this.project.deleteProject(inName);
			this.left.setLayer("projects");
		}
	},
	deletePage: function(inName) {
		if (this.application.main == inName) {
			alert(inName + "はプロジェクトホームページです。ホームページを削除することはできません。");
			return;
		}
		if (confirm(inName + 'を削除しますか？'))
			this.project.deletePage(inName);
	},
	closeClick: function() {
		if (!this.confirmAppChange("プロジェクト「${project}」を閉じますか？保存されていない内容はなくなります。"))
			return;
		this.project.closeProject();
	},
	copyProjectClick: function() {
	  if (studio.project) {
			var n = this.getNewProjectName(studio.project.projectName);
			if (n)
				this.waitForCallback("Copying project: " + studio.project.projectName + " to " + n, dojo.hitch(this.project, "copyProject", studio.project.projectName, n));
		}
	},
	importProjectClick: function(inSender) {
		var d = this.importPageDialog;
		if (d) {
			d.page.update();
		} else {
			d = this.importPageDialog = new wm.PageDialog({
				owner: studio,
				name: "importPageDialog",
				pageName: "ImportPageDialog",
				hideControls: true
			});
			// once that returns, refresh our list
			d.onClose = function(inWhy) {
				if (inWhy == "OK")
					studio.project.pagesChanged();
			};
		}
		d.show();
	},
	makeHomeClick: function() {
		this.setProjectMainPage(this.page.declaredClass);
	},
	makeSelectedHomeClick: function(inSender) {
		var n = this.projectsTree.selected;
		if (n && n.page)
			this.setProjectMainPage(n.page);
	},
	setProjectMainPage: function(inName) {
		this.application.main = inName;
		this.pagesChanged();
		this.saveProjectClick();
	},
	projectSettingsClick: function(inSender) {
		var d = this.preferencesDialog;
		if (d) {
			d.page.update();
		} else {
			this.preferencesDialog = d = new wm.PageDialog({pageName: "PreferencesPane", owner: studio, hideControls: true, contentHeight: 250, contentWidth: 500});
			d.onClose = function(inWhy) {
				if (inWhy == "OK")
					studio.updateProjectTree();
			};
		}
		d.show();
	},
	clearProjectPages: function() {
		var pages = this.project.getPageList();
		dojo.forEach(pages, function(p) {
			this.removeClassCtor(p);
		}, this);
	},
	// Note: use with extreme care, blasts away a class
	removeClassCtor: function(inName) {
		var ctor = dojo.getObject(inName);
		if (ctor)
			delete ctor;
		window[inName] = null;
	},
	getNewProjectName: function(inName) {
		var projects = this.project.getProjectList();
		var l={};
		dojo.forEach(projects, function(p) {
			l[p] = true;
		});
		return this.promptForName("project", wm.findUniqueName(inName || 'Project', [l]), projects);
	},
	confirmPageChange: function(inMessage, inNewPage) {
		var inMessage = dojo.string.substitute(inMessage, {page: '"' + this.project.pageName + '"', newPage: inNewPage});
		return this.isPageDirty() ? confirm(inMessage) : true;
	},
	confirmAppChange: function(inMessage, inNewProject) {
		var inMessage = dojo.string.substitute(inMessage, {project: '"' + this.project.projectName + '"', newProject: inNewProject});
		return this.isProjectDirty() ? confirm(inMessage) : true;
	},
	promptForName: function(inTarget, inDefault, inExistingList) {
		var
			newPrompt = '新しい${target}名を入力してください： ',
			warnExists = 'The ${target} named "${name}" already exists. Please choose a different name.',
			warnNotValid = 'The name "${name}" is not valid. Names can contain only letters, numbers, underscores, must not begin with a number, and must not be a reserved word.',
			name = prompt(dojo.string.substitute(newPrompt, {target: inTarget}), inDefault);
		if (name !== null) {
			for (var i= 0, exists = false, lcn = name.toLowerCase(), n; (n=inExistingList[i]); i++)
				if (n.toLowerCase() == lcn) {
					exists = true;
					break;
				}
			if (exists) {
				alert(dojo.string.substitute(warnExists, {target: inTarget, name: name}));
				return this.promptForName(inTarget, name, inExistingList);
			} else if (window[name] || wm.getValidJsName(name) != name) {
				alert(dojo.string.substitute(warnNotValid, {name: name}));
				return this.promptForName(inTarget, name, inExistingList);
			}
			return name;
		} else
			return null;
	},
	/* this was from before we added a "Test run" button
	getPreviewUrl: function() {
		var s = [], q="", c = wm.studioConfig;
		if (this.getUserSetting("previewDebug"))
			s.push("debug");
		if (c.previewPopup)
			s.push("popup");
		var projectPrefix = studio.projectPrefix;
		return "/" + projectPrefix + this.project.projectName + (s.length ? "/?" + s.join("&") : "");
	},
	*/
	getPreviewUrl: function(isTest) {
	    var s = [], q="", c = wm.studioConfig;
	    if (isTest)
		s.push("debug");
	    if (c.previewPopup)
		s.push("popup");
	    var projectPrefix = studio.projectPrefix;
	    return "/" + projectPrefix + this.project.projectName + (s.length ? "/?" + s.join("&") : "");
	},
	runProjectClick: function(inSender) {	    
	  this._runRequested = inSender.name;
	    var isTest = (inSender.name == "navTestBtn");
		this.saveProjectClick();
		this.deploy("プレビューの作成中...", dojo.hitch(this, function(result) {
		      this._runRequested = false;
		      wm.openUrl(this.getPreviewUrl(isTest), this.project.projectName, "_wmPreview");
		      return result;
		}));
	}
});

//=========================================================================
// Project Tree
//=========================================================================
Studio.extend({
	updateProjectTree: function() {
		var d = this.project.updateProjectList();
		d.addCallback(dojo.hitch(this, "_updateProjectTree"));
		if (this.startContainer && this.startContainer.page instanceof wm.Page) {
		  d.addCallback(dojo.hitch(this.startContainer.page, "listProjectsResult"));
		}
	},
	updateProjectTreePages: function() {
		var n = this.projectsTree.findNode({project: this.project.projectName});
		if (n) {
			n.removeChildren();
			this._updateProjectNode(n, this.project.getPageList());
			this.selectInProjectTree(this.project.projectName, this.project.pageName);
		}
	},
	_updateProjectTree: function(inResult) {
		this.projectsTree.clear();
		dojo.forEach(inResult, function(t) {
			new wm.TreeNode(studio.projectsTree.root, {
				content: t,
				project: t,
				image: "images/project_16.png",
				closed: true,
				hasChildren: true,
				initNodeChildren: dojo.hitch(this, "initProjectNodeChildren")
			});
		}, this);
		this.selectInProjectTree(this.project.projectName, this.project.pageName);
	},
	_updateProjectNode: function(inNode, inResult) {
		dojo.forEach(inResult, function(t) {
			new wm.TreeNode(inNode, {
				content: t,
				project: inNode.project,
				page: t,
				image: "images/page.png",
				hasChildren: false
			});
		});
	},
	selectInProjectTree: function(inProjectName, inPageName) {
		var n = this.projectsTree.findNode({project: inProjectName});
		if (n && inPageName) {
			n.setOpen(true);
			n = this.projectsTree.findNode({project: inProjectName, page: inPageName}, n) || n;
		}
		if (n)
			this.projectsTree.select(n);
	},
	projectsTreeDblClick: function(inSender, inNode) {
		this.openSelectedProjectPageClick();
	},
	initProjectNodeChildren: function(inNode) {
		this.pagesService.requestSync("listPages", [inNode.content], dojo.hitch(this, "_updateProjectNode", inNode))
	},
	projectsTreeSelectionChange: function(inNode) {
	  var n = this.projectsTree.selected;
	  if (!n)
	    return;

	  var sameProject =  (this.project.projectName == n.project);

	  // New page can only be created in currently open project
	  this.projectNewPageButton.setDisabled(!sameProject);

	  // Can only set the home page of the currently open project
	  this.projectSetHomePageButton.setDisabled(!sameProject);

	  if (n.page) {
	    // Can only delete pages of the currently open project
	    this.projectDeleteButton.setDisabled(!sameProject);	    ;
	  } else if (n.project) {
	    this.projectSetHomePageButton.setDisabled(true);
	    this.projectDeleteButton.setDisabled(false);
	  }

	  // Do not allow users to delete the last page of the project
	  if (n.page && n.parent.kids.length == 1) 
	    this.projectDeleteButton.setDisabled(true);
	}
});
