/*
 *  Copyright (C) 2008-2010 WaveMaker Software, Inc.
 *
 *  This file is part of the WaveMaker Client Runtime.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
dojo.provide("wm.base.widget.DojoGrid");

dojo.declare("wm.DojoGrid", wm.Control, {
	padding: 4,
	width:'100%',
	height:'200px',
	variable:null,
	dataSet:null,
	store:null,
	query:'',
	structure:'',
	customFields:'',
	dojoObj:null,
	menu:null,
	singleClickEdit:false,
	editProperties: "(Edit Properties)",
	storeGUID:1,
	selectedItem: null,
	emptySelection: true,
	selectionMode: "single",
	newColumnButton: null,
	customFieldCounter:0,
	
	init: function() {
		dojo['require']("dojox.grid.DataGrid");
		dojo['require']("dojox.grid.cells.dijit");
		dojo['require']("dojo.data.ItemFileWriteStore");
		dojo['require']("dojo.string");
		if (this.isDesignLoaded())
		{
			var js = "dijit.Dialog";
			dojo["require"](js);
		}
		
		this.inherited(arguments);
		this.selectedItem = new wm.Variable({name: "selectedItem", owner: this});
		this.updateSelectedItem(-1);
		this.setSelectionMode(this.selectionMode);
	},
	postInit: function() {
		this.inherited(arguments);
		var thisObj = this;
		dojo.addOnLoad(function(){thisObj.renderDojoObj();});
	},
	dataSetToSelectedItem: function() {
		this.selectedItem.setLiveView((this.dataSet|| 0).liveView);
		this.selectedItem.setType(this.variable && this.variable.type ? this.variable.type : "any");
	},
	setSelectedRow: function(rowIndex, isSelected) {
	  if (isSelected == undefined) 
	    isSelected = true;
	  if (isSelected) {
	    this.dojoObj.selection.select(rowIndex);
	  } else {
	    this.dojoObj.selection.setSelected(rowIndex,isSelected);
	  }
	},
	deselect: function() {
	  this.updateSelectedItem(-1);
	  this.onSelectionChange();
	},
	select: function() {
	  this.updateSelectedItem( this.getSelectedIndex());
	  this.onSelectionChange();
	},
	selectionChange: function() {
	  this.updateSelectedItem( this.getSelectedIndex());
	  this.onSelectionChange();
	},
	cellEditted: function(inValue, inRowIndex, inFieldName) {
	  // values of the selectedItem must be updated, but do NOT call a selectionChange event, as its the same selected item, just different values
	 if (this.getSelectedIndex() != inRowIndex) {
	   this.setSelectedRow(inRowIndex, true);
	 }
	  this.updateSelectedItem( this.getSelectedIndex());
	  this.onCellEditted(inValue, inRowIndex, inFieldName);
	},

	
	updateSelectedItem: function(selectedIndex) {
	  if (selectedIndex == -1) {
	    this.selectedItem.clearData();
	  } else {	    
	    var item = this.getRowData(selectedIndex);
	    var schema = this.selectedItem._dataSchema;
	    var newdata = {};
	    for (var key in schema) {
	      var data = this.dojoObj.store.getValue(item,key);
	      newdata[key] = data;
	    }
	    this.selectedItem.setData(newdata);
	  }
		this.setValue("emptySelection", !this.hasSelection());
	},
	onSelectionChange: function() {},
	onCellEditted: function(inValue, inRowIndex, inFieldName) {},
	
	getSelectedIndex: function() {
	  if (!this.dojoObj) return -1; // just in case the dojoObj hasn't been fully created yet
	  return this.dojoObj.selection.selectedIndex;
	},
	getRowData: function(rowIndex) {
	  return this.dojoObj.getItem(rowIndex);
	}, 
	findRowIndexByFieldValue: function(inFieldName, inFieldValue) {
	  var item;
	  for (var i = 0; i < this.getRowCount(); i++) {
	    item = this.dojoObj.getItem(i);
	    if (this.store.getValue(item,inFieldName) == inFieldValue) {
	      return this.dojoObj.getItemIndex(item);
	    }
	  }
	  return -1;
	},
	getCell: function(rowIndex, fieldName) {
	    var item = this.dojoObj.getItem(rowIndex);
	    return this.dojoObj.store.getValue(item,fieldName);
	},
	/* This method is flawed; tab does not work if cells are edited this way */
	editCell: function(rowIndex, fieldName) {
	  var cells = this.dojoObj.layout.cells;
	  for (var i = 0; i < cells.length; i++) {
	    if (cells[i].field == fieldName) {
	      this.dojoObj.edit.setEditCell(cells[i], rowIndex);
	      return;
	    }
	  }
	},
	setCell: function(rowIndex, fieldName, newValue) {
	  if (rowIndex < 0) {
	    console.error("setCell requires 0 or greater for row index");
	    return;
	  }

	    var item = this.dojoObj.getItem(rowIndex);
	    this.dojoObj.store.setValue(item, fieldName, newValue);
	    // Update the selectedItem variable if we just updated the selected item
	    if (this.getSelectedIndex() == rowIndex) {
	      this.updateSelectedItem(rowIndex);
	    }
	},
	deleteRow: function(rowIndex) {
	  this.updateSelectedItem(-1);
	  var item = this.getRowData(rowIndex);
	  this.dojoObj.store.deleteItem(item);
	},
	addRow: function(inFields, selectOnAdd) {
	  if (this.getRowCount() == 0 && this.variable) {
	    this.variable.setData([inFields]);
	    this.renderDojoObj();
	    if (selectOnAdd) {
	      this.setSelectedRow(0);
	      this.selectionChange(); // needs committing
	    }
	    return;
	  }
	  var data = dojo.clone(inFields);
	  data.guid = this.storeGUID;

	  var schema = this.selectedItem._dataSchema;
	  for (var key in schema) {
	    if (!(key in data)) {
	      data[key] = "";
	    }
	  }
	  var result = this.store.newItem(data);
	  this.storeGUID++;

	  if (selectOnAdd) {
	    var rowNumb = this.dojoObj.getItemIndex(result);
	    this.setSelectedRow(rowNumb);
	    this.selectionChange(); // needs committing
	  }
	  return result;
	},
	getRowCount: function() {
	  return this.dojoObj.rowCount;
	},
	hasSelection: function() {
	  return Boolean(this.getSelectedIndex() != -1);
	},

	getEmptySelection: function() {
		return !this.hasSelection();
	},
	getEmptySelection: function() {
	  var result = Boolean(!this.selectedItem.getData() || this.getSelectedIndex() == -1);
	  return result;
	},

	renderDojoObj: function() {
		if (this.variable)
			this.dataSetToSelectedItem();
		
		if (this.dojoObj != null)
		{
			this.dojoObj.destroy();
		}

		/* This stopped data grid from getting structure and stopped users from editting columns when there is no data yet.
		if (this.store == null)
			return;
		*/

		var structure = this.getStructure();
		if (structure[0].length == 0)
			structure = {};
		var props = {structure:structure, store:this.store, singleClickEdit: this.singleClickEdit, columnReordering:true, query:{}};
		this.dojoObj = new dojox.grid.DataGrid(props,dojo.doc.createElement('div'));
		this.domNode.appendChild(this.dojoObj.domNode);

		this.connectDojoEvents();
		this.setSelectionMode(this.selectionMode);
		this.dojoRenderer();
	},
	dojoRenderer: function (){
		if (!this.dojoObj)
			return;
		this.dojoObj.startup();
		if (!dojo.isIE)
			this.dojoObj.resize();
	},
	connectDojoEvents: function(){
		dojo.connect(this.dojoObj, 'onCellClick', this, 'onCellClick');
		dojo.connect(this.dojoObj, 'onClick', this, 'onClick');
		dojo.connect(this.dojoObj,'onCellDblClick', this,'onCellDblClick');
		dojo.connect(this.dojoObj,'onCellContextMenu', this, 'onCellRightClick');
		dojo.connect(this.dojoObj, "onDeselect", this, "deselect");
		dojo.connect(this.dojoObj, "onSelected", this, "select");
		dojo.connect(this.dojoObj, "onSelectionChanged", this, "selectionChange");
		dojo.connect(this.dojoObj, "onApplyCellEdit", this, "cellEditted");

		if (this.isDesignLoaded())
		{
			dojo.connect(this.dojoObj,'onMoveColumn', this, '_onMoveColumn');
			dojo.connect(this.dojoObj, 'onHeaderContextMenu', this, 'showMenuDialog');
			dojo.connect(this.dojoObj, 'onRowContextMenu', this, 'showMenuDialog');
			dojo.connect(this.dojoObj, 'onCellClick', this, 'hideMenuDialog');
		}
	},
	getDataSet: function() {
		return this.variable;
	},
	setDataSet: function (inValue, inDefault){
		this.variable = inValue;
		this.setDojoStore();
		this.setStructure();
		this.createRightClickMenu();
		if (this.isDesignLoaded())
		{
			this.updateRightClickMenu();
		}

		var thisObj = this;
		dojo.addOnLoad(function(){thisObj.renderDojoObj();});
	},
	setDojoStore: function(){
		if (this.variable == null || this.variable == '')
		{
			this.store = null;
			this.structure = '';
			return;
		}

		var storeData = {'identifier':'guid','items':[]};
		var viewFields = this.getViewFields();
		for (var i = 0; i < (this.variable.getCount() || 1); i++)
		{
		    var dataObj = this.variable.getItem(i);
			if (!dataObj)
				dataObj = null;
			var data = {};
			for (var j = 0; j < viewFields.length; j++)
			{
				var field = viewFields[j].dataIndex;
				var fieldType = viewFields[j].displayType;
				value = dataObj == null ? "" : dataObj.getValue(field);
				if (fieldType && fieldType == 'Date' && value != '')
					value = new Date(value);
				data[field] = value;
			}

			storeData.items[storeData.items.length] = dojo.mixin({},{'guid': this.storeGUID}, data);
			this.storeGUID++;
		}

		this.store = new dojo.data.ItemFileWriteStore({data: storeData});
	},
	_onMoveColumn: function(arg1, arg2, oldPos, newPos){
		var sArray = this.getStructure()[0];
		var tmp=sArray[oldPos];
		sArray.splice(oldPos,1);
		sArray.splice(newPos,0,tmp);
		this.structure = dojo.toJson([sArray]);
		wm.fire(studio.inspector, "reinspect");
	},
	getStructure: function(){
		var structure = this.structure == '' ? [[]] : dojo.fromJson(this.structure);
		for (var i = 0;i < structure[0].length;i++)
		{
			var displayType = structure[0][i].displayType || '';
			if (displayType == 'Date' )
			{
				structure[0][i].formatter = dojo.hitch(this, 'dateFormatter');
			}

			if (structure[0][i].typeValue && structure[0][i].typeValue != '' && !structure[0][i].type)
			{
				structure[0][i].type = dojo.getObject(structure[0][i].typeValue);
			}
			
			if (structure[0][i].expression && structure[0][i].expression != '' && !structure[0][i].get)
			{
				structure[0][i].get = dojo.hitch(this, 'getExpressionValue', structure[0][i].expression);
			}
			
		}
		this.onGetStructure(structure);
		return structure == '' ? [[]] : structure; 
	},
	onGetStructure: function(inStructure) {
        },
	setStructure: function(){
		if (!this.variable || this.variable == null || this.structure != '')
		{
			return;
		}
		
		this.structure = [];
		var viewFields = this.getViewFields();
		for(var i = 0; i < viewFields.length; i++)
		{
			var attr = viewFields[i].dataIndex;
			this.structure[this.structure.length] = {field:attr, name:wm.capitalize(attr), width:'auto', displayType: viewFields[i].displayType};
		};
		
		this.structure = dojo.toJson([this.structure]);
	},
	getCustomFields: function(){
		return this.customFields == '' ? [] : dojo.fromJson(this.customFields); 
	},
	setCustomFields: function(inValue){
		this.customFields = dojo.toJson(inValue);
	},
	setSelectionMode: function(inMode) {
	  this.selectionMode = inMode;
	  if (this.dojoObj) this.dojoObj.selection.setMode(this.selectionMode);
	},
	getViewFields: function(){
		var fields = [];
		if (this.variable instanceof wm.LiveVariable)
			fields = this.variable.getViewFields();
		else if (this.variable instanceof wm.Variable)
		{
			for (prop in this.variable._dataSchema)
			{
				if (this.variable._dataSchema[prop].isList)
					continue;
				fields[fields.length] = {dataIndex: prop};
			}
		}

		// User can add additional fields. Here we add custom fields.
		var customFields = this.getCustomFields();
		for (var i = 0; i < customFields.length; i++)
		{
			fields[fields.length] = customFields[i];
		}
		
		return fields;
	},
	onClick: function(evt){
	},
	onCellClick: function(evt){
	},
	onCellDblClick: function(evt){
	},
	onCellRightClick: function(evt){
	},
	dateFormatter: function(inDatum){
		var constraints = {selector:'date', formatLength:'short'}
		return dojo.date.locale.format(new Date(inDatum), constraints);
	}
});

// design only...
wm.Object.extendSchema(wm.DojoGrid, {
	variable: { ignore: 1 },
	caption:{ignore:1},
	disabled:{ignore:1},
	query: {ignore:1},
	structure:{hidden:true, group: "edit", order:10},
	customFields:{hidden:true, group: "edit", order:20},
	customFieldCounter:{hidden:true, group: "edit", order:30},
	editProperties:{group: "edit", order:40},
	store:{ignore:1},
	menu:{ignore:1},
	storeGUID:{ignore:1},
	dataValue:{ignore:1},
	selectedItem: {ignore:1, bindSource: 1, simpleBindProp: true },
	emptySelection: { ignore: true, bindSource: 1, type: "Boolean" },
	dataSet: {bindTarget: 1, group: "edit", order: 30, isList: true},
	selectionMode: {group: "edit", order: 31},
	newColumnButton: {ignore:1},
	rightClickTBody: {ignore:1}
});

wm.DojoGrid.description = "A dojo grid.";

wm.DojoGrid.extend({
	designCreate: function() {
		// if this is being created in studio, supply a default caption
		if (this._studioCreating)
			this.studioCreate();
		this.inherited(arguments);
	},
	afterPaletteDrop: function() {
		this.caption = this.caption || this.name;
		this.renderDojoObj();
	},
	createRightClickMenu: function(){
		if (!this.isDesignLoaded())
			return;
		if (this.menu != null)
			this.menu.destroy();
		this.menu = new dijit.Dialog({title:this.name + ' Properties'},dojo.doc.createElement('div'));
		return this.menu;
	},
	showMenuDialog: function(e){
		this.hideMenuDialog();
		var dialogPosition = {};
		if (e)
		{
			dialogPosition = { l: e.clientX, t:e.clientY }; 
		}
		else
		{
			var p = dojo.coords(this.domNode);
			dialogPosition = {l:p.x + 40, t: p.y + 40};
		}

		this.menu._relativePosition = dialogPosition; 
		this.menu.show();
		dijit._underlay.hide();
	},
	hideMenuDialog: function(){
		this.menu.hide();
	},
	getFieldsInStructure: function(){
		if (!this.structure || this.structure == null || this.structure == '')
			return [];
		return this.getStructure()[0];
	},
	setSingleClickEdit: function(inValue){
		this.singleClickEdit = inValue;
		if (this.dojoObj)
			this.dojoObj.singleClickEdit = this.singleClickEdit;
		this.dojoObj.render();
	},
	updateRightClickMenu: function(){
	  if (this.store == null && (!this.variable || !this.variable.type))
			return;

		this.store.fetchItemByIdentity(0);
		/*
		var viewFieldsHash = this.variable._dataSchema;
		var viewFields = [];
		for (var key in viewFieldsHash) viewFields.push(viewFieldsHash[key]);
		 */
		var viewFields = this.getViewFields();
		var fieldInStructure = this.getFieldsInStructure();

		var headerAttr = [{title:' ',width:'2%'},{title: 'Field',width:'20%'}, {title: 'Title',width:'20%'}, {title: 'Width',width:'20%'}, {title: 'Editable',width:'15%'}, {title: 'Data Expression',width:'20%'}];
		var menuTable = dojo.doc.createElement('table');
		var thead = dojo.doc.createElement('thead');
		menuTable.appendChild(thead);
		var tr = dojo.doc.createElement('tr');
		dojo.attr(tr, 'style', 'background-color:#DCDCDC');
		thead.appendChild(tr);
		dojo.forEach(headerAttr, function(attr){
			var td = dojo.doc.createElement('td');
			dojo.attr(td,'width',attr.width);
			dojo.attr(td,'align','center');
			td.innerHTML = attr.title;
			tr.appendChild(td);
		});

		this.rightClickTBody = dojo.doc.createElement('tbody');
		menuTable.appendChild(this.rightClickTBody);
		this.menu.containerNode.appendChild(menuTable);
		for(var j = 0; j < viewFields.length; j++)
		{
			var attr = viewFields[j].dataIndex;
			var structureObj = null;
			for(var i = 0; i < fieldInStructure.length; i++)
			{
				if (fieldInStructure[i].field == attr)
				{
					structureObj = fieldInStructure[i];
					break;
				}
			}

			this.addFieldToRightClickPanel(structureObj, viewFields[j], this.rightClickTBody);
		}

		this.paintNewColumnButton();
	},
	addFieldToRightClickPanel: function(structureObj, fieldObj, tbody){
		var widgetArray = [];
		var attr = fieldObj.dataIndex;
		var isCustomField = false;
		if (fieldObj.isCustomField)
			isCustomField = fieldObj.isCustomField;
		
		var tr1 = dojo.doc.createElement('tr');
		tbody.appendChild(tr1);
		var td1 = dojo.doc.createElement('td');
		tr1.appendChild(td1);
		var params = {};
		var widget = new dijit.form.CheckBox(params, dojo.doc.createElement('div'));
		if (structureObj != null)
			widget.attr('checked', true);
		td1.appendChild(widget.domNode);
		dojo.connect(widget, 'onChange', dojo.hitch(this, 'updateStructure', attr, 'cb', isCustomField, tr1));
		widgetArray[widgetArray.length] = widget;
		
		var td2 = dojo.doc.createElement('td');
		tr1.appendChild(td2);
		var params = {value: attr, readOnly: true, disabled: true};
		var widget = new dijit.form.TextBox(params, dojo.doc.createElement('div'));
		td2.appendChild(widget.domNode);
		widgetArray[widgetArray.length] = widget;
		
		var td3 = dojo.doc.createElement('td');
		tr1.appendChild(td3);
		var params = {value: structureObj != null ? structureObj.name : wm.capitalize(attr)};
		var widget = new dijit.form.TextBox(params, dojo.doc.createElement('div'));
		td3.appendChild(widget.domNode);
		dojo.connect(widget, 'onChange',dojo.hitch(this, 'updateStructure', attr, 'name', isCustomField, tr1));
		widgetArray[widgetArray.length] = widget;
		
		var td4 = dojo.doc.createElement('td');
		tr1.appendChild(td4);
		var params = {value: structureObj != null ? structureObj.width : 'auto'};
		var widget = new dijit.form.TextBox(params, dojo.doc.createElement('div'));
		td4.appendChild(widget.domNode);
		dojo.connect(widget, 'onChange',dojo.hitch(this, 'updateStructure', attr, 'width', isCustomField, tr1));
		widgetArray[widgetArray.length] = widget;
		
		var td5 = dojo.doc.createElement('td');
		tr1.appendChild(td5);
		var params = {autoComplete: false,store: this.getGridEditStore()};
		var widget = new dijit.form.FilteringSelect(params, dojo.doc.createElement('div'));
		widgetArray[widgetArray.length] = widget;
		if (structureObj != null && structureObj.typeValue && structureObj.typeValue != '')
		{
			widget.attr('value',structureObj.typeValue);
		}
		
		td5.appendChild(widget.domNode);
		dojo.connect(widget, 'onChange',dojo.hitch(this, 'updateStructure', attr, 'editType', isCustomField, tr1));

		var td6 = dojo.doc.createElement('td');
		tr1.appendChild(td6);
		var params = {value: structureObj != null ? structureObj.expression : ''};
		var widget = new dijit.form.TextBox(params, dojo.doc.createElement('div'));
		td6.appendChild(widget.domNode);
		dojo.connect(widget, 'onChange',dojo.hitch(this, 'updateStructure', attr, 'expression', isCustomField, tr1));
		widgetArray[widgetArray.length] = widget;
		
		return widgetArray;
	},
	paintNewColumnButton: function(){
		if (this.newColumnButton)
			this.newColumnButton.destroy();
		var div = dojo.doc.createElement('div');
		this.menu.containerNode.appendChild(div);
		this.newColumnButton = new dijit.form.Button({label: "Add Column", onClick: dojo.hitch(this, 'addNewColumn'), style:'padding:5px;'});
		this.newColumnButton.placeAt(div);
	},
	addNewColumn: function(){
		var customFields = this.getCustomFields();
		var customField = {dataIndex: 'customField' + this.customFieldCounter, isCustomField: true, expression: ''};
		this.customFieldCounter++;
		customFields[customFields.length] = customField;
		this.setCustomFields(customFields);
		var widgetArray = this.addFieldToRightClickPanel(null, customField, this.rightClickTBody);
		widgetArray[0].attr('checked', true);
	},
	updateStructure: function(attr, type, isCustomField, trObj, inValue){
		var structure = this.getStructure();
		if (type == 'cb')
		{
			if (inValue)
			{
				structure[0].push({field:attr, name: wm.capitalize(attr), width:'auto'});
			}
			else
			{
				for (var i = structure[0].length - 1; i >= 0; i--)
				{
					var fieldObj = structure[0][i];
					if (fieldObj.field == attr)
					{
						structure[0].splice(i,1);
					}
				}
				
				if (isCustomField)
				{
					trObj.parentNode.removeChild(trObj);
					var customFields = this.getCustomFields();
					for (i = 0; i < customFields.length; i++)
					{
						if (customFields[i].dataIndex == attr)
						{
							customFields.splice(i,1);
							this.setCustomFields(customFields);
							break;
						}
					}
				}
			}
		}
		else if (type == 'editType')
		{
			var isEditable = false;
			var inValueObj = null;
			if (inValue != '')
			{
				inValueObj = dojo.getObject(inValue);
				isEditable = true;
			}
			else
			{
				inValueObj = '';
			}
			
			for (var i = 0; i < structure[0].length; i++)
			{
				if (structure[0][i].field == attr)
				{
					structure[0][i]['type'] = inValueObj;
					structure[0][i]['typeValue'] = inValue;
					structure[0][i]['editable'] = isEditable;
					break;
				}
			}
		}
		else
		{
			for (var i = 0; i < structure[0].length; i++)
			{
				if (structure[0][i].field == attr)
				{
					structure[0][i][type] = inValue;
					break;
				}
			}
		}
		
		this.structure = dojo.toJson(structure);
		this.dojoObj.attr('structure', this.getStructure());
		this.dojoObj.render();
	},
	getExpressionValue: function(exp, idx, dataObj){
		var expValue = '..';
		if (!dataObj)
			return expValue;
		try
		{
			expValue = wm.expression.getValue(exp, dataObj);
		}
		catch(e)
		{
			// User entered a wrong expression and so we will return value as ".." for the grid column.
		}

		return expValue;
	},
	getGridEditStore: function(){
		var fieldOptions = [{name:'',value:''},
		                    {name:'Text',value:'dojox.grid.cells._Widget'},
		                    {name:'Number',value:'dojox.grid.cells.NumberTextBox'},
		                    {name:'Date',value:'dojox.grid.cells.DateTextBox'},
		                    {name:'Checkbox',value:'dojox.grid.cells.Bool'}];
		var dataItems = new Array();
		for ( var i = 0; i < fieldOptions.length; i++)
		{
			dataItems[dataItems.length] = fieldOptions[i];
		}		
		
		var storeData = {
		   identifier: 'value',
		   label: 'name',
		   value: 'value',
		   items: dataItems
		};

		return new dojo.data.ItemFileWriteStore({data: storeData});	
	},
	makePropEdit: function(inName, inValue, inDefault) {
		switch (inName) {
		case "selectionMode":
			return makeSelectPropEdit(inName, inValue, ["single", "multiple", "extended", "none"], inDefault);
		case "editProperties":
			return makeReadonlyButtonEdit(inName, inValue, inDefault);
		}
		return this.inherited(arguments);
	},
	editProp: function(inName, inValue, inInspector) {
		switch (inName) {
			case "editProperties":
				return this.showMenuDialog();
		}
		return this.inherited(arguments);
	}
});

